local FileDialog = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.FileDialog")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;

FileDialog = ui.Window:new()

function FileDialog.create()
	local o
	o = FileDialog:new()
	o.width = 400
	o.height = 300
	o.path = ""
	o.deep = 0
	o.dialogtype = "open"
	o.params["filename"] = ""
	o.fileext = ".lua"
	return o
end


function FileDialog.createOpen()
	local o
	o = FileDialog.create()
	o.dialogtype = "open"
	return o
end

function FileDialog.createSave()
	local o
	o = FileDialog.create()
	o.dialogtype = "save"
	return o
end

function FileDialog:open()
	self:updateFilelist()
	print("FileDialog:open")
	ui.Window.open(self)
end



function FileDialog:onchange()
	if (self.params["Path"] ~= nil) then
--		print("Path changed?")
		if (self.params["Path"] ~= self.path) then
			self.path = self.params["Path"]
			self.params["Path"] = nil
			self.deep = 0
			self:updateFilelist()
		end
	end
end

function FileDialog:updateFilelist()
	local uilist = {}
	local nativefs = commonapi._native.fs
	local realpath = nativefs.realpath(self.path)

--	print(nativefs.realpath(self.path))

	local entries = {}
	if (realpath ~= nil) then
		realpath = string.gsub(realpath, '\\', '/')..'/'
		for filename, fflags in nativefs.dir(realpath) do
			if (fflags == 2) then
				entries[#entries+1] = {
					isDir = true,
					name = filename,
					realpath = realpath..filename
				}
			else
				entries[#entries+1] = {
					isDir = false,
					name = filename,
					realpath = realpath..filename
				}
			end
		end
	end
	table.sort(entries, function(a, b)
		-- (a.isDir < b.isDir) or (a.isDir == b.isDir and a.name < b.name)
		return (a.isDir == true and b.isDir == false) or (a.isDir == b.isDir and a.name < b.name)
	end)

	uilist[#uilist+1] = {
		typ = "TextInput",
		id = "Path",
		default = self.path,
		changeonenter = true,
--		onclick = function(item, win)
--			commonapi._native.utils.openfolder(win.path)
--		end
	}

	uilist[#uilist+1] = {
		typ = "_ScrollRegionStart",
		id = "ScrollRegion",
		border = true,
		size = {
			0, -60
		}
	}

	if (realpath == nil) then
		uilist[#uilist+1] = {
			typ = "Label",
			id = "NoPath",
			caption = _("Folder not found")
		}
	end

--	if (self.deep > 0) then
		uilist[#uilist+1] = {
			typ = "_Selectable",
			id = "__back__",
			caption = "..",
			data = entry,
			ondblclick = function(item, win)
				win.path = string.gsub(win.path, '\\', '/')
				win.path = string.gsub(win.path, "(/)$", "")
				win.path = string.gsub(win.path, "(/[^/]+)$", "")
				win.deep = win.deep - 1
				win:updateFilelist()
			end
		}
--	end
	local showEntry = true
	for _, entry in ipairs(entries) do
		if (entry.isDir) then
			uilist[#uilist+1] = {
				typ = "_Selectable",
				id = entry.name,
				caption = "[Dir] "..entry.name,
				data = entry,
				ondblclick = function(item, win)
					if (win.path:sub(-1) == "/") then
						win.path = win.path .. item.data.name
					else
						win.path = win.path .. "/"..item.data.name
					end
					win.deep = win.deep + 1
					win:updateFilelist()
				end,
			}
		else
			if (self.fileext ~= nil) then
				if (type(self.fileext) == 'table') then
					showEntry = false
					for extindex, ext in ipairs(self.fileext) do
						if (entry.name:sub(-#ext) == ext) then
							showEntry = true
							break
						end
					end
				else
					showEntry =	entry.name:sub(-#self.fileext) == self.fileext
				end
			end

			if (showEntry) then
				uilist[#uilist+1] = {
					typ = "_Selectable",
					id = "file_"..entry.name,
					caption = "[File] "..entry.name,
					data = entry,
					onclick = function(item, win)
						win.params["filename"] = item.data.name
					end,
				}
			end
		end
--		uilist[#uilist+1] = {
--			typ = "Separator"
--		}
	end

	uilist[#uilist+1] = {
		typ = "_ScrollRegionEnd",
		id = "ScrollRegionEnd"
	}

	uilist[#uilist+1] = {
		typ = "Separator"
	}

	uilist[#uilist+1] = {
		typ = "Label",
		caption = _("Filename:"),
	}

	uilist[#uilist+1] = {
		typ = "TextInput",
		id = "filename",
		sameline = true,
	}
	
	if (self.dialogtype == "open") then
		uilist[#uilist+1] = {
			typ = "Button",
			id = "load",
			caption = _("Load"),
			onclick = function(item, win)
				if (win.params["filename"] == "") then
					return
				end
				if (win.loadFile ~= nil) then
					local filename = win.path
					if (filename:sub(-1) ~= "/") then
						filename = filename .. "/"
					end
					filename = filename .. win.params["filename"]
					win:loadFile(filename)
				end
			end
		}
	else
		uilist[#uilist+1] = {
			typ = "Button",
			id = "save",
			caption = _("Save"),
			onclick = function(item, win)
				if (win.params["filename"] == "") then
					return
				end
				if (win.saveFile ~= nil) then
					local filename = win.path
					if (filename:sub(-1) ~= "/") then
						filename = filename .. "/"
					end
					filename = filename .. win.params["filename"]

					if (self.fileext ~= nil) then
						if (filename:sub(-#self.fileext) ~= self.fileext) then
							filename = filename..self.fileext
						end
					end
					win:saveFile(filename)
				end
			end
		}

	end



	self.uilist = uilist

end
return FileDialog